library IEEE;
use IEEE.std_logic_1164.all;
use IEEE.std_logic_arith.all;
use IEEE.std_logic_unsigned.all;

entity avalon_video is
  port (
    avs_s1_clk        : in  std_logic;
    avs_s1_reset_n    : in  std_logic;
    avs_s1_read       : in  std_logic;
    avs_s1_write      : in  std_logic;
    avs_s1_chipselect : in  std_logic;
    avs_s1_address    : in  std_logic_vector(6 downto 0);
    avs_s1_readdata   : out std_logic_vector(31 downto 0);
    avs_s1_writedata  : in  std_logic_vector(31 downto 0);
	
	reset_n	  : in std_logic;					  -- More reset!
	CLK_25	  : in std_logic;
    VGA_CLK   : out std_logic;                    -- Clock
    VGA_HS    : out std_logic;                    -- H_SYNC
    VGA_VS    : out std_logic;                    -- V_SYNC
    VGA_BLANK : out std_logic;                    -- BLANK
    VGA_SYNC  : out std_logic;                    -- SYNC
    VGA_R     : out std_logic_vector(9 downto 0); -- Red[9:0]
    VGA_G     : out std_logic_vector(9 downto 0); -- Green[9:0]
    VGA_B     : out std_logic_vector(9 downto 0)  -- Blue[9:0]
  );
end avalon_video;

architecture rtl of avalon_video is

signal clk : std_logic;

--Declare our various RAMs and ROMs
type maze_ram_line_type is array(40 downto -1) of std_logic;
type maze_ram_type is array(-1 to 30) of maze_ram_line_type;
type maze_sprite_ram_type is array(0 to 7) of std_logic_vector(255 downto 0);

type score_ram_line_type is array(2 downto 0) of std_logic_vector(3 downto 0);
type score_ram_type is array(1 downto 0) of score_ram_line_type;

type sprite_ram_line_type is array(0 to 15) of std_logic_vector(0 to 15);
type dot_sprite_ram_type is array(0 to 1) of sprite_ram_line_type;
type num_sprite_ram_type is array(0 to 10) of sprite_ram_line_type;

signal maze_sprite_ram: maze_sprite_ram_type;
signal dot_sprite_ram: dot_sprite_ram_type;
signal num_sprite_ram: num_sprite_ram_type;
signal score_ram: score_ram_type;
signal RAM : maze_ram_type;
signal ram_address, display_address : std_logic_vector(1 downto 0);

constant HTOTAL       : integer := 800;
constant HSYNC        : integer := 96;
constant HBACK_PORCH  : integer := 48;
constant HACTIVE      : integer := 640;
constant HFRONT_PORCH : integer := 16;
  
constant VTOTAL       : integer := 525;
constant VSYNC        : integer := 2;
constant VBACK_PORCH  : integer := 33;
constant VACTIVE      : integer := 480;
constant VFRONT_PORCH : integer := 10;

-- Signals for the video controller
signal Hcount : std_logic_vector(9 downto 0);  -- Horizontal position (0-800)
signal Vcount : std_logic_vector(9 downto 0);  -- Vertical position (0-524)
signal EndOfLine, EndOfField : std_logic;

signal vga_hblank, vga_hsync,
 vga_vblank, vga_vsync : std_logic;  -- Sync. signals

signal visible_xpos: std_logic_vector(9 downto 0);
signal visible_ypos: std_logic_vector(9 downto 0);

signal sprite_x: std_logic_vector(3 downto 0);
signal sprite_y: std_logic_vector(3 downto 0);

signal maze_x: std_logic_vector(5 downto 0);
signal maze_y: std_logic_vector(4 downto 0);
signal maze : std_logic;
signal print_char : std_logic;
signal maze_line_from_RAM : maze_ram_line_type;
signal left_maze, right_maze, up_maze, down_maze, downleft_maze, upleft_maze, upright_maze, downright_maze: std_logic;

signal current_maze_sprite_from_RAM : std_logic_vector(255 downto 0);
signal current_dot_sprite_from_RAM : sprite_ram_line_type;
signal current_char_sprite_from_RAM : sprite_ram_line_type;

begin

  clk <= avs_s1_clk;
--  reset_n <= avs_s1_reset_n;
  ram_address <= avs_s1_address;
  visible_xpos <= Hcount - HSYNC - HBACK_PORCH;
  maze_x <= visible_xpos(9 downto 4);
  sprite_x <= visible_xpos(3 downto 0);
  left_maze <= RAM(conv_integer(maze_y))(conv_integer(maze_x)-1);
  right_maze <= RAM(conv_integer(maze_y))(conv_integer(maze_x)+1);
  up_maze <= RAM(conv_integer(maze_y)-1)(conv_integer(maze_x));
  down_maze <= RAM(conv_integer(maze_y)+1)(conv_integer(maze_x));
  downleft_maze <= RAM(conv_integer(maze_y)+1)(conv_integer(maze_x)-1);
  downright_maze <= RAM(conv_integer(maze_y)+1)(conv_integer(maze_x)+1);
  upleft_maze <= RAM(conv_integer(maze_y)-1)(conv_integer(maze_x)-1);
  upright_maze <= RAM(conv_integer(maze_y)-1)(conv_integer(maze_x)+1);
  score_ram(0)(0) <= "0010"; score_ram(1)(2) <= "0100";

  MazeSprite: process (clk)
  begin
	  if up_maze = '1' and down_maze = '1' and left_maze = '0' and right_maze = '1' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(6);
  	  elsif up_maze = '1' and down_maze = '1' and left_maze = '1' and right_maze = '0' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(6);

	  elsif up_maze = '1' and right_maze = '1' and upright_maze = '1'  and downleft_maze = '0' and down_maze = '0' and left_maze = '0' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(1);
	  elsif up_maze = '1' and right_maze = '1' and upright_maze = '0' and downleft_maze = '1' and down_maze = '1' and left_maze = '1' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(1);

	  elsif down_maze = '1' and right_maze = '1' and downright_maze = '1' and left_maze = '0' and upleft_maze = '0' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(2);
	  elsif down_maze = '1' and right_maze = '1' and downright_maze = '0' and left_maze = '1' and upleft_maze = '1' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(2);

	  elsif up_maze = '1' and left_maze = '1' and upleft_maze = '1' and downright_maze = '0' and down_maze = '0' and right_maze = '0' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(3);
	  elsif up_maze = '1' and left_maze = '1' and upleft_maze = '0' and upleft_maze = '0' and downright_maze = '1' and down_maze = '1' and downright_maze = '1' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(3);

	  elsif left_maze = '1' and down_maze = '1' and upright_maze = '0' and right_maze = '0' and downleft_maze = '1' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(4);
	  elsif left_maze = '1' and down_maze = '1' and upright_maze = '1' and right_maze = '1' and downleft_maze = '0' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(4);

	  elsif left_maze = '1' and right_maze = '1' and up_maze = '0' and down_maze = '1' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(5);
	  elsif left_maze = '1' and right_maze = '1' and up_maze = '1' and down_maze = '0' then
		current_maze_sprite_from_RAM <= maze_sprite_ram(5);
	  
      else
		current_maze_sprite_from_RAM <= maze_sprite_ram(7);
	  end if;

	
	  if maze_x = "000010" and maze_y = "00010" then
		print_char <= '1';
		current_char_sprite_from_RAM <= num_sprite_ram(1);
	  elsif maze_x = "100010" and maze_y = "00010" then
		print_char <= '1';
		current_char_sprite_from_RAM <= num_sprite_ram(2);
	  elsif maze_x = "000010" and maze_y = "00011" then
		print_char <= '1';
		current_char_sprite_from_RAM <= num_sprite_ram(conv_integer(score_ram(0)(0)));
	  elsif maze_x = "000011" and maze_y = "00011" then
		print_char <= '1';
		current_char_sprite_from_RAM <= num_sprite_ram(conv_integer(score_ram(0)(1)));
	  elsif maze_x = "000100" and maze_y = "00011" then
		print_char <= '1';
		current_char_sprite_from_RAM <= num_sprite_ram(conv_integer(score_ram(0)(2)));
	  elsif maze_x = "100010" and maze_y = "00011" then
		print_char <= '1';
		current_char_sprite_from_RAM <= num_sprite_ram(conv_integer(score_ram(1)(0)));
	  elsif maze_x = "100011" and maze_y = "00011" then
		print_char <= '1';
		current_char_sprite_from_RAM <= num_sprite_ram(conv_integer(score_ram(1)(1)));
	  elsif maze_x = "100100" and maze_y = "00011" then
		print_char <= '1';
		current_char_sprite_from_RAM <= num_sprite_ram(conv_integer(score_ram(1)(2)));
	  else
		print_char <= '0';
	  end if;
  end process;

  process (clk)
  begin
	--all of these registers end up being reduced, so no worries :-)  I'll try to clean it up later, though.
    if clk'event and clk = '1' then
	  RAM(-1)<=	"111111111111111111111111111111111111111111";
	  RAM(0) <= "111111111111111111111111111111111111111111";
	  RAM(1) <=	"111111110000000000001100000000000011111111";
	  RAM(2) <=	"111111110111101111101101111101111011111111";
	  RAM(3) <= "111111110111101111101101111101111011111111";
	  RAM(4) <= "111111110111101111101101111101111011111111";
	  RAM(5) <= "111111110000000000000000000000000011111111";
	  RAM(6) <= "111111110111101101111111101101111011111111";
	  RAM(7) <=	"111111110111101101111111101101111011111111";
	  RAM(8) <= "111111110000001100001100001100000011111111";
	  RAM(9)<=	"111111111111101111101101111101111111111111";
	  RAM(10)<= "111111111111101111101101111101111111111111";
	  RAM(11)<= "111111111111101100000000001101111111111111";
	  RAM(12)<= "111111111111101101111111101101111111111111";
	  RAM(13)<= "111111111111101101111111101101111111111111";
	  RAM(14)<= "111111110000000001111111100000000011111111";
	  RAM(15)<= "111111111111101101111111101101111111111111";
	  RAM(16)<=	"111111111111101101111111101101111111111111";
	  RAM(17)<=	"111111111111101100000000001101111111111111";
	  RAM(18)<= "111111111111101101111111101101111111111111";
	  RAM(19)<= "111111111111101101111111101101111111111111";
	  RAM(20)<= "111111110000000000001100000000000011111111";
	  RAM(21)<= "111111110111101111101101111101111011111111";
	  RAM(22)<=	"111111110111101111101101111101111011111111";
	  RAM(23)<= "111111110001100000000000000001100011111111";
	  RAM(24)<=	"111111111101101101111111101101101111111111";
	  RAM(25)<= "111111111101101101111111101101101111111111";
	  RAM(26)<= "111111110000001100001100001100000011111111";
	  RAM(27)<= "111111110111111111101101111111111011111111";
	  RAM(28)<= "111111110111111111101101111111111011111111";
	  RAM(29)<=	"111111110000000000000000000000000011111111";
	  RAM(30)<=	"111111111111111111111111111111111111111111";

	  maze_sprite_ram(0) <= "1111111111111111000000000000000011111111111111110000000000000000111111111111111100000000000000001111111111111111000000000000000011111111111111110000000000000000111111111111111100000000000000001111111111111111000000000000000011111111111111110000000000000000";
	  maze_sprite_ram(1) <= "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000111111000000000011111110000000000000001100000000000000011000000000000001100000000000000110000000000000011000000000000001100000000000000110000000";
	  maze_sprite_ram(2) <= "0000000110000000000000011000000000000001100000000000000110000000000000011000000000000001100000000000001100000000111111100000000011111100000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000";
      maze_sprite_ram(3) <= "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000011111100000000011111110000000011000000000000011000000000000001100000000000000110000000000000011000000000000001100000000000000110000000";
	  maze_sprite_ram(4) <= "0000000110000000000000011000000000000001100000000000000110000000000000011000000000000001100000000000000011000000000000000111111100000000001111110000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000";
	  maze_sprite_ram(5) <= "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000111111111111111111111111111111110000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000";
	  maze_sprite_ram(6) <= "0000000110000000000000011000000000000001100000000000000110000000000000011000000000000001100000000000000110000000000000011000000000000001100000000000000110000000000000011000000000000001100000000000000110000000000000011000000000000001100000000000000110000000";
	  maze_sprite_ram(7) <= "0000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000000";

	  dot_sprite_ram(0)(0) <= "0000000000000000";
	  dot_sprite_ram(0)(1) <= "0000000000000000";
	  dot_sprite_ram(0)(2) <= "0000000000000000";
	  dot_sprite_ram(0)(3) <= "0000000000000000";
	  dot_sprite_ram(0)(4) <= "0000000000000000";
	  dot_sprite_ram(0)(5) <= "0000000000000000";
	  dot_sprite_ram(0)(6) <= "0000000000000000";
	  dot_sprite_ram(0)(7) <= "0000000110000000";
	  dot_sprite_ram(0)(8) <= "0000000110000000";
	  dot_sprite_ram(0)(9) <= "0000000000000000";
	  dot_sprite_ram(0)(10) <="0000000000000000";
	  dot_sprite_ram(0)(11) <="0000000000000000";
	  dot_sprite_ram(0)(12) <="0000000000000000";
	  dot_sprite_ram(0)(13) <="0000000000000000";
	  dot_sprite_ram(0)(14) <="0000000000000000";
	  dot_sprite_ram(0)(15) <="0000000000000000";

	  num_sprite_ram(0)(0) <= "0000000000000000";	
	  num_sprite_ram(0)(1) <= "0000011111100000";
	  num_sprite_ram(0)(2) <= "0000011111100000";
	  num_sprite_ram(0)(3) <= "0001100001111000";
	  num_sprite_ram(0)(4) <= "0001100001111000";
	  num_sprite_ram(0)(5) <= "0111100000011110";
	  num_sprite_ram(0)(6) <= "0111100000011110";
	  num_sprite_ram(0)(7) <= "0111100000011110";
	  num_sprite_ram(0)(8) <= "0111100000011110";
	  num_sprite_ram(0)(9) <= "0111100000011110";
	  num_sprite_ram(0)(10) <="0111100000011110";
	  num_sprite_ram(0)(11) <="0001111000011000";
	  num_sprite_ram(0)(12) <="0001111000011000";
	  num_sprite_ram(0)(13) <="0000011111100000";
	  num_sprite_ram(0)(14) <="0000011111100000";
	  num_sprite_ram(0)(15) <="0000000000000000";


	  num_sprite_ram(1)(0) <=	"0000000000000000";	
	  num_sprite_ram(1)(1) <=	"0000001111000000";
	  num_sprite_ram(1)(2) <=	"0000001111000000";
	  num_sprite_ram(1)(3) <=	"0000111111000000";
	  num_sprite_ram(1)(4) <=	"0000111111000000";
	  num_sprite_ram(1)(5) <=	"0000001111000000";
	  num_sprite_ram(1)(6) <=	"0000001111000000";
	  num_sprite_ram(1)(7) <=	"0000001111000000";
	  num_sprite_ram(1)(8) <=	"0000001111000000";
	  num_sprite_ram(1)(9) <=	"0000001111000000";
	  num_sprite_ram(1)(10) <=	"0000001111000000";
	  num_sprite_ram(1)(11) <=	"0000001111000000";
	  num_sprite_ram(1)(12) <=	"0000001111000000";
	  num_sprite_ram(1)(13) <=	"0011111111111100";
	  num_sprite_ram(1)(14) <=	"0011111111111100";
	  num_sprite_ram(1)(15) <=	"0000000000000000";

	  num_sprite_ram(2)(0) <=	"0000000000000000";
	  num_sprite_ram(2)(1) <=	"0001111111111000";	
	  num_sprite_ram(2)(2) <=	"0001111111111000";
	  num_sprite_ram(2)(3) <=	"0111100000011110";
	  num_sprite_ram(2)(4) <=	"0111100000011110";
	  num_sprite_ram(2)(5) <=	"0000000001111110";
	  num_sprite_ram(2)(6) <=	"0000000001111110";
	  num_sprite_ram(2)(7) <=	"0000011111111000";
	  num_sprite_ram(2)(8) <=	"0000011111111000";
	  num_sprite_ram(2)(9) <=	"0001111111100000";
	  num_sprite_ram(2)(10) <=	"0001111111100000";
	  num_sprite_ram(2)(11) <=	"0111111000000000";
	  num_sprite_ram(2)(12) <=	"0111111000000000";
	  num_sprite_ram(2)(13) <=	"0111111111111110";
	  num_sprite_ram(2)(14) <=	"0111111111111110";
	  num_sprite_ram(2)(15) <=	"0000000000000000";

	  num_sprite_ram(3)(0) <= "0000000000000000";
	  num_sprite_ram(3)(1) <= "0000011111111000";
	  num_sprite_ram(3)(2) <= "0000011111111000";
	  num_sprite_ram(3)(3) <= "0001100001111000";
	  num_sprite_ram(3)(4) <= "0001100001111000";
	  num_sprite_ram(3)(5) <= "0000000001111000";
	  num_sprite_ram(3)(6) <= "0000000001111000";
	  num_sprite_ram(3)(7) <= "0000000111100000";
	  num_sprite_ram(3)(8) <= "0000000111100000";
	  num_sprite_ram(3)(9) <= "0000000001111110";
	  num_sprite_ram(3)(10) <="0000000001111110";
	  num_sprite_ram(3)(11) <="0110000000011110";
	  num_sprite_ram(3)(12) <="0110000000011110";
	  num_sprite_ram(3)(13) <="0001111111111000";
	  num_sprite_ram(3)(14) <="0001111111111000";
	  num_sprite_ram(3)(15) <="0000000000000000";

	  num_sprite_ram(4)(0) <= "0000000000000000";
	  num_sprite_ram(4)(1) <= "0000000111111000";
	  num_sprite_ram(4)(2) <= "0000000111111000";
	  num_sprite_ram(4)(3) <= "0000011111111000";
	  num_sprite_ram(4)(4) <= "0000011111111000";
	  num_sprite_ram(4)(5) <= "0001111001111000";
	  num_sprite_ram(4)(6) <= "0001111001111000";
	  num_sprite_ram(4)(7) <= "0111100001111000";
	  num_sprite_ram(4)(8) <= "0111100001111000";
	  num_sprite_ram(4)(9) <= "0111111111111110";
	  num_sprite_ram(4)(10) <="0000000001111000";
	  num_sprite_ram(4)(11) <="0000000001111000";
	  num_sprite_ram(4)(12) <="0000000001111000";
	  num_sprite_ram(4)(13) <="0000000001111000";
	  num_sprite_ram(4)(14) <="0000000001111000";
	  num_sprite_ram(4)(15) <="0000000000000000";

	  num_sprite_ram(5)(0) <= "0000000000000000";
	  num_sprite_ram(5)(1) <= "0111111111111000";
	  num_sprite_ram(5)(2) <= "0111111111111000";
	  num_sprite_ram(5)(3) <= "0111100000000000";
	  num_sprite_ram(5)(4) <= "0111100000000000";
	  num_sprite_ram(5)(5) <= "0111111111111000";
	  num_sprite_ram(5)(6) <= "0111111111111000";
	  num_sprite_ram(5)(7) <= "0000000000011110";
	  num_sprite_ram(5)(8) <= "0000000000011110";
	  num_sprite_ram(5)(9) <= "0000000000011110";
	  num_sprite_ram(5)(10) <="0000000000011110";
	  num_sprite_ram(5)(11) <="0111100000011110";
	  num_sprite_ram(5)(12) <="0111100000011110";
	  num_sprite_ram(5)(13) <="0001111111111000";
	  num_sprite_ram(5)(14) <="0001111111111000";
	  num_sprite_ram(5)(15) <="0000000000000000";

	  num_sprite_ram(6)(0) <= "0000000000000000";
	  num_sprite_ram(6)(1) <= "0001111111111000";
	  num_sprite_ram(6)(2) <= "0001111111111000";
	  num_sprite_ram(6)(3) <= "0111100000000000";
	  num_sprite_ram(6)(4) <= "0111100000000000";
	  num_sprite_ram(6)(5) <= "0111111111111000";
	  num_sprite_ram(6)(6) <= "0111111111111000";
	  num_sprite_ram(6)(7) <= "0111100000011110";
	  num_sprite_ram(6)(8) <= "0111100000011110";
	  num_sprite_ram(6)(9) <= "0111100000011110";
	  num_sprite_ram(6)(10) <="0111100000011110";
	  num_sprite_ram(6)(11) <="0111100000011110";
	  num_sprite_ram(6)(12) <="0111100000011110";
	  num_sprite_ram(6)(13) <="0001111111111000";
	  num_sprite_ram(6)(14) <="0001111111111000";
	  num_sprite_ram(6)(15) <="0000000000000000";

	  num_sprite_ram(7)(0) <= "0000000000000000";
	  num_sprite_ram(7)(1) <= "0111111111111110";
	  num_sprite_ram(7)(2) <= "0111111111111110";
	  num_sprite_ram(7)(3) <= "0111100000011110";
	  num_sprite_ram(7)(4) <= "0111100000011110";
	  num_sprite_ram(7)(5) <= "0000000001111000";
	  num_sprite_ram(7)(6) <= "0000000001111000";
	  num_sprite_ram(7)(7) <= "0000000111100000";
	  num_sprite_ram(7)(8) <= "0000000111100000";
	  num_sprite_ram(7)(9) <= "0000011110000000";
	  num_sprite_ram(7)(10) <="0000011110000000";
	  num_sprite_ram(7)(11) <="0000011110000000";
	  num_sprite_ram(7)(12) <="0000011110000000";
	  num_sprite_ram(7)(13) <="0000011110000000";
	  num_sprite_ram(7)(14) <="0000011110000000";
	  num_sprite_ram(7)(15) <="0000000000000000";

	  num_sprite_ram(8)(0) <= "0000000000000000";
	  num_sprite_ram(8)(1) <= "0001111111100000";
	  num_sprite_ram(8)(2) <= "0001111111100000";
	  num_sprite_ram(8)(3) <= "0111100000011000";
	  num_sprite_ram(8)(4) <= "0111100000011000";
	  num_sprite_ram(8)(5) <= "0111111000011000";
	  num_sprite_ram(8)(6) <= "0111111000011000";
	  num_sprite_ram(8)(7) <= "0001111111100000";
	  num_sprite_ram(8)(8) <= "0001111111100000";
	  num_sprite_ram(8)(9) <= "0110000111111110";
	  num_sprite_ram(8)(10) <="0110000111111110";
	  num_sprite_ram(8)(11) <="0110000000011110";
	  num_sprite_ram(8)(12) <="0110000000011110";
	  num_sprite_ram(8)(13) <="0001111111111000";
	  num_sprite_ram(8)(14) <="0001111111111000";
	  num_sprite_ram(8)(15) <="0000000000000000";

	  num_sprite_ram(9)(0) <= "0000000000000000";
	  num_sprite_ram(9)(1) <= "0001111111111000";
	  num_sprite_ram(9)(2) <= "0001111111111000";
	  num_sprite_ram(9)(3) <= "0111100000011110";
	  num_sprite_ram(9)(4) <= "0111100000011110";
	  num_sprite_ram(9)(5) <= "0111100000011110";
	  num_sprite_ram(9)(6) <= "0111100000011110";
	  num_sprite_ram(9)(7) <= "0001111111111110";
	  num_sprite_ram(9)(8) <= "0001111111111110";
	  num_sprite_ram(9)(9) <= "0000000000011110";
	  num_sprite_ram(9)(10) <="0000000000011110";
	  num_sprite_ram(9)(11) <="0000000001111000";
	  num_sprite_ram(9)(12) <="0000000001111000";
	  num_sprite_ram(9)(13) <="0001111111100000";
	  num_sprite_ram(9)(14) <="0001111111100000";
	  num_sprite_ram(9)(15) <="0000000000000000";

	  num_sprite_ram(10)(0) <= "0000000000000000";
	  num_sprite_ram(10)(1) <= "0000000000000000";
	  num_sprite_ram(10)(2) <= "0000000000000000";
	  num_sprite_ram(10)(3) <= "0000000000000000";
	  num_sprite_ram(10)(4) <= "0000000000000000";
	  num_sprite_ram(10)(5) <= "0000000000000000";
	  num_sprite_ram(10)(6) <= "0000000000000000";
	  num_sprite_ram(10)(7) <= "0000000000000000";
	  num_sprite_ram(10)(8) <= "0000000000000000";
	  num_sprite_ram(10)(9) <= "0000000000000000";
	  num_sprite_ram(10)(10) <="0000000000000000";
	  num_sprite_ram(10)(11) <="0000000000000000";
	  num_sprite_ram(10)(12) <="0000000000000000";
	  num_sprite_ram(10)(13) <="0000000000000000";
	  num_sprite_ram(10)(14) <="0000000000000000";
	  num_sprite_ram(10)(15) <="0000000000000000";

	  if avs_s1_reset_n = '0' then
        avs_s1_readdata <= (others => '0');
        display_address <= (others => '0');        
--        counter <= (others => '0');        
--        counter_delay <= (others => '1');
      else
        if avs_s1_chipselect = '1' then
          if avs_s1_read = '1' then
--            avs_s1_readdata <= RAM(conv_integer(ram_address));
            avs_s1_readdata <= (others => '0');
--          elsif avs_s1_write = '1' then
--		;
--         RAM(conv_integer(ram_address)) <= avs_s1_writedata;
--			rectangle_hstart <= avs_s1_writedata(9 downto 0);
          end if;
        end if;
      end if;
    end if;
  end process;

	process (clk)
	begin
		if clk'event and clk = '1' then
--			maze <= maze_line_from_RAM(conv_integer(maze_x));
			maze <= RAM(conv_integer(maze_y))(conv_integer(maze_x));
		end if;
	end process;
	
  HCounter : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      Hcount <= (others => '0');
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        Hcount <= (others => '0');
      else
        Hcount <= Hcount + 1;
      end if;      
    end if;
  end process HCounter;

  EndOfLine <= '1' when Hcount = HTOTAL - 1 else '0';
 
  VCounter: process (clk_25, reset_n)
  begin
    if reset_n = '1' then
      Vcount <= (others => '0');
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        if EndOfField = '1' then
          Vcount <= (others => '0');
        else
          Vcount <= Vcount + 1;
	      visible_ypos <= Vcount - VSYNC - VBACK_PORCH;
		  sprite_y <= visible_ypos(3 downto 0);
		  if maze_y /= visible_ypos(8 downto 4) then
	  		  maze_y <= visible_ypos(8 downto 4);
			  maze_line_from_RAM <= RAM(conv_integer(maze_y));
		  end if;
        end if;
      end if;
    end if;
  end process VCounter;

  EndOfField <= '1' when Vcount = VTOTAL - 1 else '0';

  -- State machines to generate HSYNC, VSYNC, HBLANK, and VBLANK

  HSyncGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      vga_hsync <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        vga_hsync <= '1';
      elsif Hcount = HSYNC - 1 then
        vga_hsync <= '0';
      end if;      
    end if;
  end process HSyncGen;
  
  HBlankGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      vga_hblank <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if Hcount = HSYNC + HBACK_PORCH then
        vga_hblank <= '0';
      elsif Hcount = HSYNC + HBACK_PORCH + HACTIVE then
        vga_hblank <= '1';
      end if;      
    end if;
  end process HBlankGen;

  VSyncGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      vga_vsync <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine ='1' then
        if EndOfField = '1' then
          vga_vsync <= '1';
        elsif Vcount = VSYNC - 1 then
          vga_vsync <= '0';
        end if;
      end if;      
    end if;
  end process VSyncGen;

  VBlankGen : process (clk_25, reset_n)
  begin    
    if reset_n = '1' then
      vga_vblank <= '1';
    elsif clk_25'event and clk_25 = '1' then
      if EndOfLine = '1' then
        if Vcount = VSYNC + VBACK_PORCH - 1 then
          vga_vblank <= '0';
        elsif Vcount = VSYNC + VBACK_PORCH + VACTIVE - 1 then
          vga_vblank <= '1';
        end if;
      end if;      
    end if;
  end process VBlankGen;

  -- Registered video signals going to the video DAC

  VideoOut: process (clk_25, reset_n)
  begin
    if reset_n = '1' then
      VGA_R <= "0000000000";
      VGA_G <= "0000000000";
      VGA_B <= "0000000000";
    elsif clk_25'event and clk_25 = '1' then 	--it is here that we determine which sprites take precedence over each other
												--this is really important stuff!
      if maze = '1' then
		if print_char = '1' then --if we should be printing some number or text instead of the maze:
			VGA_R <= (others => current_char_sprite_from_ram(conv_integer(sprite_y))(conv_integer(sprite_x)));
			VGA_G <= (others => current_char_sprite_from_ram(conv_integer(sprite_y))(conv_integer(sprite_x)));
			VGA_B <= (others => current_char_sprite_from_ram(conv_integer(sprite_y))(conv_integer(sprite_x)));
		else
	        VGA_R <= "0000000000";
	        VGA_G <= "0000000000";
	        VGA_B <= (others => current_maze_sprite_from_ram(conv_integer(sprite_x) + conv_integer(sprite_y&"0000")));
		end if;
      elsif vga_hblank = '0' and vga_vblank ='0' then
        VGA_R <= (others => dot_sprite_ram(0)(conv_integer(sprite_y))(conv_integer(sprite_x)));
        VGA_G <= (others => dot_sprite_ram(0)(conv_integer(sprite_y))(conv_integer(sprite_x)));
        VGA_B <= "0000000000";
      else
        VGA_R <= "0000000000";
        VGA_G <= "0000000000";
        VGA_B <= "0000000000";    
      end if;
    end if;
  end process VideoOut;

  VGA_CLK <= clk_25;
  VGA_HS <= not vga_hsync;
  VGA_VS <= not vga_vsync;
  VGA_SYNC <= '0';
  VGA_BLANK <= not (vga_hsync or vga_vsync);
end rtl;
